@@ -32,6 +32,8 @@
     typedef __u16 u16;
     typedef __u8 u8;
 #endif
+/* Avoid redefinition of struct sysinfo with musl libc */
+#define _LINUX_SYSINFO_H
 #include <linux/ethtool.h>
 
 /* The minimum number of CPUs allocated in a cpu_set_t */
@@ -455,7 +457,7 @@ psutil_users(PyObject *self, PyObject *args) {
             (float)ut->ut_tv.tv_sec,  // tstamp
             py_user_proc              // (bool) user process
         );
-    if (! py_tuple)
+        if (! py_tuple)
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
