@@ -16,25 +16,24 @@
 #include <net/if.h>
 
 #ifdef PSUTIL_SUNOS10
-#include "arch/solaris/v10/ifaddrs.h"
+    #include "arch/solaris/v10/ifaddrs.h"
 #else
-#include <ifaddrs.h>
+    #include <ifaddrs.h>
 #endif
 
-#ifdef __linux
-#include <netdb.h>
-#include <linux/if_packet.h>
-#endif  // end linux
-
-#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__NetBSD__)
-#include <netdb.h>
-#include <netinet/in.h>
-#include <net/if_dl.h>
-#endif
-
-#if defined(__sun)
-#include <netdb.h>
-#include <sys/sockio.h>
+#if defined(PSUTIL_LINUX)
+    #include <netdb.h>
+    #include <linux/if_packet.h>
+#elif defined(PSUTIL_BSD) || defined(PSUTIL_OSX)
+    #include <netdb.h>
+    #include <netinet/in.h>
+    #include <net/if_dl.h>
+    #include <sys/sockio.h>
+    #include <net/if_media.h>
+    #include <net/if.h>
+#elif defined(PSUTIL_SUNOS)
+    #include <netdb.h>
+    #include <sys/sockio.h>
 #endif
 
 
@@ -49,7 +48,12 @@ psutil_posix_getpriority(PyObject *self, PyObject *args) {
 
     if (! PyArg_ParseTuple(args, "l", &pid))
         return NULL;
+
+#ifdef PSUTIL_OSX
+    priority = getpriority(PRIO_PROCESS, (id_t)pid);
+#else
     priority = getpriority(PRIO_PROCESS, pid);
+#endif
     if (errno != 0)
         return PyErr_SetFromErrno(PyExc_OSError);
     return Py_BuildValue("i", priority);
@@ -67,7 +71,12 @@ psutil_posix_setpriority(PyObject *self, PyObject *args) {
 
     if (! PyArg_ParseTuple(args, "li", &pid, &priority))
         return NULL;
+
+#ifdef PSUTIL_OSX
+    retval = setpriority(PRIO_PROCESS, (id_t)pid, priority);
+#else
     retval = setpriority(PRIO_PROCESS, pid, priority);
+#endif
     if (retval == -1)
         return PyErr_SetFromErrno(PyExc_OSError);
     Py_RETURN_NONE;
@@ -112,14 +121,13 @@ psutil_convert_ipaddr(struct sockaddr *addr, int family) {
             return Py_BuildValue("s", buf);
         }
     }
-#ifdef __linux
+#ifdef PSUTIL_LINUX
     else if (family == AF_PACKET) {
         struct sockaddr_ll *lladdr = (struct sockaddr_ll *)addr;
         len = lladdr->sll_halen;
         data = (const char *)lladdr->sll_addr;
     }
-#endif
-#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__NetBSD__)
+#elif defined(PSUTIL_BSD) || defined(PSUTIL_OSX)
     else if (addr->sa_family == AF_LINK) {
         // Note: prior to Python 3.4 socket module does not expose
         // AF_LINK so we'll do.
@@ -254,8 +262,11 @@ psutil_net_if_mtu(PyObject *self, PyObject *args) {
     char *nic_name;
     int sock = 0;
     int ret;
-    int mtu;
+#ifdef PSUTIL_SUNOS10
+    struct lifreq lifr;
+#else
     struct ifreq ifr;
+#endif
 
     if (! PyArg_ParseTuple(args, "s", &nic_name))
         return NULL;
@@ -264,14 +275,22 @@ psutil_net_if_mtu(PyObject *self, PyObject *args) {
     if (sock == -1)
         goto error;
 
+#ifdef PSUTIL_SUNOS10
+    strncpy(lifr.lifr_name, nic_name, sizeof(lifr.lifr_name));
+    ret = ioctl(sock, SIOCGIFMTU, &lifr);
+#else
     strncpy(ifr.ifr_name, nic_name, sizeof(ifr.ifr_name));
     ret = ioctl(sock, SIOCGIFMTU, &ifr);
+#endif
     if (ret == -1)
         goto error;
     close(sock);
-    mtu = ifr.ifr_mtu;
 
-    return Py_BuildValue("i", mtu);
+#ifdef PSUTIL_SUNOS10
+    return Py_BuildValue("i", lifr.lifr_mtu);
+#else
+    return Py_BuildValue("i", ifr.ifr_mtu);
+#endif
 
 error:
     if (sock != 0)
@@ -322,11 +341,7 @@ error:
 /*
  * net_if_stats() OSX/BSD implementation.
  */
-#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__NetBSD__)
-
-#include <sys/sockio.h>
-#include <net/if_media.h>
-#include <net/if.h>
+#if defined(PSUTIL_BSD) || defined(PSUTIL_OSX)
 
 int psutil_get_nic_speed(int ifm_active) {
     // Determine NIC speed. Taken from:
@@ -359,7 +374,7 @@ int psutil_get_nic_speed(int ifm_active) {
                 case(IFM_1000_SX):  // 1000BaseSX - multi-mode fiber
                 case(IFM_1000_LX):  // 1000baseLX - single-mode fiber
                 case(IFM_1000_CX):  // 1000baseCX - 150ohm STP
-#if defined(IFM_1000_TX) && !defined(__OpenBSD__)
+#if defined(IFM_1000_TX) && !defined(PSUTIL_OPENBSD)
                 // FreeBSD 4 and others (but NOT OpenBSD) -> #define IFM_1000_T in net/if_media.h
                 case(IFM_1000_TX):
 #endif
@@ -532,7 +547,7 @@ PsutilMethods[] = {
      "Retrieve NIC MTU"},
     {"net_if_flags", psutil_net_if_flags, METH_VARARGS,
      "Retrieve NIC flags"},
-#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__NetBSD__)
+#if defined(PSUTIL_BSD) || defined(PSUTIL_OSX)
     {"net_if_duplex_speed", psutil_net_if_duplex_speed, METH_VARARGS,
      "Return NIC stats."},
 #endif
@@ -557,6 +572,7 @@ psutil_posix_traverse(PyObject *m, visitproc visit, void *arg) {
     return 0;
 }
 
+
 static int
 psutil_posix_clear(PyObject *m) {
     Py_CLEAR(GETSTATE(m)->error);
@@ -591,7 +607,7 @@ void init_psutil_posix(void)
     PyObject *module = Py_InitModule("_psutil_posix", PsutilMethods);
 #endif
 
-#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__) || defined(__sun) || defined(__NetBSD__)
+#if defined(PSUTIL_BSD) || defined(PSUTIL_OSX) || defined(PSUTIL_SUNOS)
     PyModule_AddIntConstant(module, "AF_LINK", AF_LINK);
 #endif
 
