@@ -19,7 +19,6 @@ HANDLE g_hThread = NULL;
 PUNICODE_STRING g_pNameBuffer = NULL;
 ULONG g_dwSize = 0;
 ULONG g_dwLength = 0;
-PVOID g_fiber = NULL;
 
 
 PVOID
@@ -74,9 +73,9 @@ psutil_get_open_files_ntqueryobject(long dwPid, HANDLE hProcess) {
     PSYSTEM_HANDLE_TABLE_ENTRY_INFO_EX  hHandle = NULL;
     DWORD                               i = 0;
     BOOLEAN                             error = FALSE;
-    PyObject*                           pyListFiles = NULL;
-    PyObject*                           pyFilePath = NULL;
     DWORD                               dwWait = 0;
+    PyObject*                           py_retlist = NULL;
+    PyObject*                           py_path = NULL;
 
     if (g_initialized == FALSE)
         psutil_get_open_files_init(TRUE);
@@ -97,8 +96,8 @@ psutil_get_open_files_ntqueryobject(long dwPid, HANDLE hProcess) {
     }
 
     // Py_BuildValue raises an exception if NULL is returned
-    pyListFiles = PyList_New(0);
-    if (pyListFiles == NULL) {
+    py_retlist = PyList_New(0);
+    if (py_retlist == NULL) {
         error = TRUE;
         goto cleanup;
     }
@@ -138,8 +137,7 @@ psutil_get_open_files_ntqueryobject(long dwPid, HANDLE hProcess) {
         hHandle = &pHandleInfo->Handles[i];
 
         // Check if this hHandle belongs to the PID the user specified.
-        if (hHandle->UniqueProcessId != (HANDLE)dwPid ||
-            hHandle->ObjectTypeIndex != HANDLE_TYPE_FILE)
+        if (hHandle->UniqueProcessId != (HANDLE)dwPid)
             goto loop_cleanup;
 
         if (!DuplicateHandle(hProcess,
@@ -207,9 +205,9 @@ psutil_get_open_files_ntqueryobject(long dwPid, HANDLE hProcess) {
                    g_pNameBuffer->Buffer);
             */
 
-            pyFilePath = PyUnicode_FromWideChar(g_pNameBuffer->Buffer,
+            py_path = PyUnicode_FromWideChar(g_pNameBuffer->Buffer,
                                                 g_pNameBuffer->Length/2);
-            if (pyFilePath == NULL) {
+            if (py_path == NULL) {
                 /*
                 printf("[%d] PyUnicode_FromWideChar (%#x): %#x \n",
                        dwPid,
@@ -220,7 +218,7 @@ psutil_get_open_files_ntqueryobject(long dwPid, HANDLE hProcess) {
                 goto loop_cleanup;
             }
 
-            if (PyList_Append(pyListFiles, pyFilePath)) {
+            if (PyList_Append(py_retlist, py_path)) {
                 /*
                 printf("[%d] PyList_Append (%#x): %#x \n",
                        dwPid,
@@ -233,8 +231,8 @@ psutil_get_open_files_ntqueryobject(long dwPid, HANDLE hProcess) {
         }
 
 loop_cleanup:
-        Py_XDECREF(pyFilePath);
-        pyFilePath = NULL;
+        Py_XDECREF(py_path);
+        py_path = NULL;
 
         if (g_pNameBuffer != NULL)
             HeapFree(GetProcessHeap(), 0, g_pNameBuffer);
@@ -263,13 +261,13 @@ cleanup:
     pHandleInfo = NULL;
 
     if (error) {
-        Py_XDECREF(pyListFiles);
-        pyListFiles = NULL;
+        Py_XDECREF(py_retlist);
+        py_retlist = NULL;
     }
 
     LeaveCriticalSection(&g_cs);
 
-    return pyListFiles;
+    return py_retlist;
 }
 
 
@@ -278,12 +276,13 @@ psutil_NtQueryObject() {
     DWORD dwWait = 0;
 
     if (g_hThread == NULL)
-        g_hThread = CreateThread(NULL,
-                                 0,
-                                 (LPTHREAD_START_ROUTINE)psutil_NtQueryObjectThread,
-                                 NULL,
-                                 0,
-                                 NULL);
+        g_hThread = CreateThread(
+            NULL,
+            0,
+            psutil_NtQueryObjectThread,
+            NULL,
+            0,
+            NULL);
     if (g_hThread == NULL)
         return GetLastError();
 
@@ -300,11 +299,6 @@ psutil_NtQueryObject() {
         WaitForSingleObject(g_hThread, INFINITE);
         CloseHandle(g_hThread);
 
-        // Cleanup Fiber
-        if (g_fiber != NULL)
-            DeleteFiber(g_fiber);
-        g_fiber = NULL;
-
         g_hThread = NULL;
     }
 
@@ -312,12 +306,8 @@ psutil_NtQueryObject() {
 }
 
 
-void
-psutil_NtQueryObjectThread() {
-    // Prevent the thread stack from leaking when this
-    // thread gets terminated due to NTQueryObject hanging
-    g_fiber = ConvertThreadToFiber(NULL);
-
+DWORD WINAPI
+psutil_NtQueryObjectThread(LPVOID lpvParam) {
     // Loop infinitely waiting for work
     while (TRUE) {
         WaitForSingleObject(g_hEvtStart, INFINITE);
@@ -343,8 +333,8 @@ psutil_get_open_files_getmappedfilename(long dwPid, HANDLE hProcess) {
     HANDLE                              hMap = NULL;
     DWORD                               i = 0;
     BOOLEAN                             error = FALSE;
-    PyObject*                           pyListFiles = NULL;
-    PyObject*                           pyFilePath = NULL;
+    PyObject*                           py_retlist = NULL;
+    PyObject*                           py_path = NULL;
     ULONG                               dwSize = 0;
     LPVOID                              pMem = NULL;
     TCHAR                               pszFilename[MAX_PATH+1];
@@ -359,8 +349,8 @@ psutil_get_open_files_getmappedfilename(long dwPid, HANDLE hProcess) {
     }
 
     // Py_BuildValue raises an exception if NULL is returned
-    pyListFiles = PyList_New(0);
-    if (pyListFiles == NULL) {
+    py_retlist = PyList_New(0);
+    if (py_retlist == NULL) {
         error = TRUE;
         goto cleanup;
     }
@@ -400,8 +390,7 @@ psutil_get_open_files_getmappedfilename(long dwPid, HANDLE hProcess) {
         hHandle = &pHandleInfo->Handles[i];
 
         // Check if this hHandle belongs to the PID the user specified.
-        if (hHandle->UniqueProcessId != (HANDLE)dwPid ||
-            hHandle->ObjectTypeIndex != HANDLE_TYPE_FILE)
+        if (hHandle->UniqueProcessId != (HANDLE)dwPid)
             goto loop_cleanup;
 
         if (!DuplicateHandle(hProcess,
@@ -444,7 +433,8 @@ psutil_get_open_files_getmappedfilename(long dwPid, HANDLE hProcess) {
             goto loop_cleanup;
         }
 
-        dwSize = GetMappedFileName(GetCurrentProcess(), pMem, pszFilename, MAX_PATH);
+        dwSize = GetMappedFileName(
+            GetCurrentProcess(), pMem, pszFilename, MAX_PATH);
         if (dwSize == 0) {
             /*
             printf("[%d] GetMappedFileName (%#x): %#x \n",
@@ -464,8 +454,8 @@ psutil_get_open_files_getmappedfilename(long dwPid, HANDLE hProcess) {
                pszFilename);
         */
 
-        pyFilePath = PyUnicode_FromWideChar(pszFilename, dwSize);
-        if (pyFilePath == NULL) {
+        py_path = PyUnicode_FromWideChar(pszFilename, dwSize);
+        if (py_path == NULL) {
             /*
             printf("[%d] PyUnicode_FromStringAndSize (%#x): %#x \n",
                    dwPid,
@@ -476,7 +466,7 @@ psutil_get_open_files_getmappedfilename(long dwPid, HANDLE hProcess) {
             goto loop_cleanup;
         }
 
-        if (PyList_Append(pyListFiles, pyFilePath)) {
+        if (PyList_Append(py_retlist, py_path)) {
             /*
             printf("[%d] PyList_Append (%#x): %#x \n",
                    dwPid,
@@ -488,8 +478,8 @@ psutil_get_open_files_getmappedfilename(long dwPid, HANDLE hProcess) {
         }
 
 loop_cleanup:
-        Py_XDECREF(pyFilePath);
-        pyFilePath = NULL;
+        Py_XDECREF(py_path);
+        py_path = NULL;
 
         if (pMem != NULL)
             UnmapViewOfFile(pMem);
@@ -524,9 +514,9 @@ cleanup:
     pHandleInfo = NULL;
 
     if (error) {
-        Py_XDECREF(pyListFiles);
-        pyListFiles = NULL;
+        Py_XDECREF(py_retlist);
+        py_retlist = NULL;
     }
 
-    return pyListFiles;
+    return py_retlist;
 }
