@@ -308,6 +308,12 @@ psutil_winservice_query_status(PyObject *self, PyObject *args) {
     // right size.
     QueryServiceStatusEx(hService, SC_STATUS_PROCESS_INFO, NULL, 0,
                          &bytesNeeded);
+    if (GetLastError() == ERROR_MUI_FILE_NOT_FOUND) {
+        // Also services.msc fails in the same manner, so we return an
+        // empty string.
+        CloseServiceHandle(hService);
+        return Py_BuildValue("s", "");
+    }
     if (GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
         PyErr_SetFromWindowsErr(0);
         goto error;
@@ -376,6 +382,12 @@ psutil_winservice_query_descr(PyObject *self, PyObject *args) {
     bytesNeeded = 0;
     QueryServiceConfig2(hService, SERVICE_CONFIG_DESCRIPTION, NULL, 0,
                         &bytesNeeded);
+    if (GetLastError() == ERROR_MUI_FILE_NOT_FOUND) {
+        // Also services.msc fails in the same manner, so we return an
+        // empty string.
+        CloseServiceHandle(hService);
+        return Py_BuildValue("s", "");
+    }
     if (GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
         PyErr_SetFromWindowsErr(0);
         goto error;
@@ -403,6 +415,7 @@ psutil_winservice_query_descr(PyObject *self, PyObject *args) {
         goto error;
 
     free(scd);
+    CloseServiceHandle(hService);
     return py_retstr;
 
 error:
